#!/usr/bin/perl -w
###############################################################################
## MSS /etc/hosts.allow Updater script v1.7
## Created by phyber- on EFNet
## This script is made for sites that have dynamic IPs and dyndns names 
## associated wtih them.
## It should update the /etc/hosts.allow file with a sites IPs when it is run.
## Designed to be ran from crontab, so you don't have to look at 
## /etc/hosts.allow ever again. Unless I screwed up.
## Add the following to crontab to make it run every three hours.
## * */3 * * * /glftpd/bin/mssdns.pl >/dev/null 2>&1
## Or you can change it to your own time period, whatever.
## Oh, and you'll also need the Net::DNS perl module, you can install that
## with the following command: 'perl -MCPAN -e 'install Net::DNS'
## Enjoy
###############################################################################
use strict;
use Net::DNS;

## Configuration
my @sites; # Don't remove
my @services = ("portmap", "lockd", "mountd", "statd"); # Change if needed
my $hallow = "/etc/hosts.allow"; # Location of hosts.allow file
my @exportfs = ("/usr/sbin/exportfs", "-r"); # ("command", "arg", "arg");
my $state = "/glftpd/etc/mss-dns.state"; # state file, stores old IPs

# Site list, you should add sites following this format.
$sites[0] = "site.dns";
$sites[1] = "other.dns";
#$sites[2] = "broken.address.for.testing";

# Show some debugging info, useful for first run.
# You probably want to disable this.
# Note: first run will always throw out some concatination errors.
my $debug = 1;
## End of user configuration

###############################################################################
## The script, you shouldn't need to change anything south of here.          ##
###############################################################################
my %ipnew = (); # Don't remove
my %ipold = (); # Don't remove
my $needupdate = 0; # Don't change or remove

# Load old IPs from the state file
if (-e $state) {
	open FILE, "< $state"
		or die "Couldn't open $state: $!\n";
	while (<FILE>) {
		chomp;
		my ($s, $i) = split(/:/, $_);
		if ($debug) { print "Loaded state for $s: $i\n"; }
		$ipold{"$s"} = $i;
	}
	close FILE;
} else {
	$needupdate = 1;
}

# Get the IP address of each site listed
sub get_ips {
	foreach my $site (@sites) {
		my $res = Net::DNS::Resolver->new;
		my $query = $res->search("$site");

		if ($query) {
			foreach my $rr ($query->answer) {
				next unless $rr->type eq "A";
				if ($debug) { print "$site: ", $rr->address, "\n"; }
				$ipnew{"$site"} = $rr->address;
				}
			} else {
				warn "Query failed: ", $res->errorstring, "\n";
				if ($debug) { print "Couldn't resolve, keeping old IP...\n"; }
				$ipnew{"$site"} = $ipold{"$site"};
		}
	}
}

# Check that the IPs actually changed
sub check_ips {
	my ($site) = @_;
	# Check if new IP matches old IP.
	return unless($ipold{"$site"} and $ipnew{"$site"});
	if ($ipold{"$site"} eq $ipnew{"$site"}) {
		if ($debug) { print "No update needed for $site: ".$ipold{"$site"}." == ".$ipnew{"$site"}."\n"; }
	} else {
		if ($debug) { print "Updating: ".$ipold{"$site"}." didn't match ".$ipnew{"$site"}."\n"; }
		$needupdate = 1;
	}
}

# Write the hosts.allow file
sub update_ips {
	my @iplist;
	foreach my $site (@sites) {
		push @iplist, $ipnew{"$site"};
	}
	
	my $ips = join(', ', @iplist);
	if ($debug) { print "hosts.allow IPs: $ips\n"; }

	if (-e $hallow) {
		if ($debug) { print "Removing old $hallow...\n"; }
		unlink "$hallow";
	}
	
	open FILE, ">$hallow"
		or die "Couldn't open $hallow: $!\n";
	
	if ($debug) { print "Creating $hallow with content...\n"; }
	foreach my $service (@services) {
		if ($debug) { print "$service: $ips\n"; }
		print FILE "$service: $ips\n";
	}
	close FILE;

	if (-e $state) {
		if ($debug) { print "Removing old $state...\n"; }
		unlink "$state";
	}

	open FILE, ">$state"
		or die "Couldn't open $state: $!\n";
	
	if ($debug) { print "Creating $state with content...\n"; }
	foreach my $site (sort keys %ipnew) {
		my $ip = $ipnew{"$site"};
		if ($debug) { print "$site:$ip\n"; }
		print FILE "$site:$ip\n";
	}
	close FILE;

	# Finally, run exportfs
	system(@exportfs);
}

get_ips;

foreach my $site (@sites) {
	check_ips($site);
}

if ($needupdate) {
	update_ips;
}
